/*
 PTmail 

 Author: Falke Bruinsma
 Illinois Institute of Technology, Chicago, IL

 This program replaces (at least to some degree) VAX/VMS MAIL. It is supposed
 to look and feel somewhat like Elm, however a few things are different. I 
 think the biggest difference is the fact that you exit PTmail with 'x' and
 not with 'q'uit at least when you want to save your changes. Elm is exactly
 the otherway around for who knows what reason.
 The previous version of PTmail was simply a 'wrap-around' for the MAIL program
 itself. Very clumsy but it worked to some degree. This version, is a complete
 rewrite of all MAIL routines and VAX/VMS MAIL is no longer used as a 
 foundation.

 February 18, 1994	v3.0
   Initial coding done after 2 weekends. First weekend I worked from Friday
 night 6:00 til Sunday morning 10:00am _straight_. The second weekend I 
 only worked on Saturday from 10:00am til 9:00pm. After those two programming
 session the major layout of the program was done and I had people test it.
 One week later after fixing quite some bugs and optimizing the code I made
 it public.
 February 21, 1994	
  -Optimized screen redraws by deleting unnecessay info and just leaving
 the windows in the back blank. 
  -Added CTRL-Z as a nice way of exiting the program
  -Added a function to strip the garbage off the 'from' field when display
 the msgs in the folder
September 15, 1994	v4.0
  -bugs have been fixed.. one in particular that blew up the program
 when moving messages between different folders.
  -rewrite the part on how reread folder works.. (now I just exit the
  handlefolder and restart it with the same foldername)
  -If the "copy send", "copy reply" or "copy forward" flags are set you will 
 actually get a copy of the message send out
  -Distribution lists (@filename.ext) will work both at the To: and CC: prompt
  -(o)ptions menu is now listed in the help menu
  -When forwarding a message lines are wrapped correctly  
  -Messages marked as "D"eleted first and then "M"oved wot
 deleted
  -Removed unnecessary screen updates 
  -Internet aliases are handled correctly
*/

#include "mail.h"
#include <descrip.h>
#include <maildef.h>
#include <smgdef.h>
#include <ssdef.h>
#include <stdio.h>
#include <nam.h>
#include <string.h>
#include <ctype.h>

				/* Global variables used for the windows */
long file_context = 0, message_context = 0, user_context = 0, send_context = 0;
long pboard_id, body_id, command_id, keyboard_id, error_id, full_screen_id;
long bottom_id, title_id, help1_id, help2_id, input_id;
				/* Global variable pointing to a linked list */
alias_info *aliases=0;

/*
 close_message will close the MAIL file
*/
void close_message()
{
  itemlist null_list[] = {0, 0, 0, 0};

  mail$message_end(message_context, null_list, null_list);
  message_context = 0;
}                               /* End of close_message */

/*
 open_message will open the MAIL file and get the message context 
*/
void open_message()
{
  itemlist null_list[] = {0, 0, 0, 0};
  itemlist itmlst_begin[3] = {
    {sizeof(file_context), MAIL$_MESSAGE_FILE_CTX, &file_context, 0},
    {0, MAIL$_NOSIGNAL, 0, 0},
    {0, 0, 0, 0}
  };
  
  if (message_context != 0) close_message();
  if (!vms_ok(mail$message_begin(&message_context, itmlst_begin, null_list))) {
    fprintf(stderr, "Error in mail$mailfile_begin [init_mail]\n");
    exit(1);
  }				/* End of errorchecking */
}				/* End of open_message() */

/*
 open_mail will initialize all kinds of context variables used by the
 MAIL routines
*/
void open_mail()
{
  long indexed = 0;
  itemlist null_list[] = {0, 0, 0, 0};
  itemlist out_list[2] = {
    {sizeof(indexed), MAIL$_MAILFILE_INDEXED, &indexed, 0},
    {0, 0, 0, 0}
  };

  if (!vms_ok(mail$mailfile_begin(&file_context, null_list, null_list))) {
    fprintf(stderr, "Error in mail$mailfile_open [init_mail]\n");
    exit(1);
  }				/* End of errorchecking */

  if (!vms_ok(mail$mailfile_open(&file_context, null_list, out_list))) {
    fprintf(stderr, "Error in mail$mailfile_open [init_mail]\n");
    exit(1);
  }				/* End of errorechecking */
  if (indexed != 1) {		/* not an indexed file?? oh oh.. */
    fprintf(stderr, "Mail file not indexed (ISAM) [init_mail]\n");
    exit(1);
  }				/* End of if (index.. */
  if (!vms_ok(mail$user_begin(&user_context, null_list, null_list))) {
    fprintf(stderr, "Error in mail$user_begin [init_mail]\n");
    exit(1);
  }				/* End of errorchecking */
}				/* End of open_mail() */

/*
 close_mail does the opposite of open_mail :-)
*/
void close_mail()
{ 
  itemlist null_list[] = {0, 0, 0, 0};

  mail$user_end(&user_context, null_list, null_list);
  mail$mailfile_end(&file_context, null_list, null_list);
  mail$mailfile_close(&file_context, null_list, null_list);
}				/* End of close_mail() */
    
/* 
 Main procedure which isnt actually the main function since the program
 itself evolves around the handle_folder function. 
   main()
       \-handle_folder()         
                     \-handle_msgs()
                                 \-read_msg()
*/
main(int argc, char *argv[])
{
  struct list folders = {0, 0, 0};
  struct list *tmp;
  char temp[256];
  user_info profile;
  char foldername[256] = "";		/* Always put user in NEWMAIL */
  short terminator, changed = FALSE;

  printf("Loading PTmail v4.0 by Falke Bruinsma (Falke_Bruinsma@iit.edu)\n");
  printf("Opening mail file...");
  open_mail();
  printf("\r                     \r");
  printf("Reading mail profile...");
  read_profile(&profile);
  printf("\r                              \r");
  printf("Reading mail folders...");
  read_folders(&profile, &folders);
  printf("\r                       \r");
  printf("Reading alias file...");
  read_aliases();
  printf("\r                     \r");
  delete_tempfiles();		/* Delete possible old temp files */
  init_screen(&profile);	/* Create all virtual windows etc */
  cursor_off(); 		/* Turn the cursor off */
  if (profile.new_messages == 0) 
    strcpy(foldername, "MAIL");	/* If there is no newmail go to MAIL */
  else 
    strcpy(foldername, "NEWMAIL");
  if (argc > 1) 		/* if an arguament is giving.. */
    upcase(foldername, argv[1]);	/* .. then use that folder */
  do {
   open_message();
				/* get commands and do stuff :) */
   terminator = handle_folder(&profile, &folders, foldername, aliases, &changed);
   switch (terminator) {	/* What key got returned? */
    case 99: 			/* "c" Change folder */
      if (changed) {
         clear(&command_id);
         center(&command_id, "Reading folders... Please Wait");
         read_folders(&profile, &folders);
      }
				/* get a new foldername */
      if (get_folder_name(&profile,&folders, temp, 0)) 
        strcpy(foldername, temp);
      break;
    case 82:			/* "R" reread, use same folder */
     break;
   }				/* End of switch(.. */
  } while ((terminator != 113) && 	/* quit */
          (terminator != 120) &&       	/* exit */
          (terminator != 26));		/* CTRL-Z */
  cursor_on();			/* Turn the cursor back on */
  restore_screen();		/* Clear the screen */
  if (profile.auto_purge && profile.waste_messages > 10)
    purge_mail();
  close_mail();
}				/* End of main() */
